<?php

// 

include 'common.php';

define('THEME', basename(realpath('../../../')));

define('MANIFEST',               ROOT . '/manifest');
define('THEMES',                 ROOT . '/themes');
define('THEME_ROOT',             THEMES . DS . THEME);
define('THEME_PREVIEW',          THEME_ROOT . '/__editor__/data/preview');
define('THEME_RUNTIME',          THEME_ROOT . '/__editor__/runtime');
define('THEME_RUNTIME_PREVIEW',  THEME_ROOT . '/__editor__/runtime/__preview__');
define('THEME_RUNTIME_MANIFEST', THEME_ROOT . '/__editor__/runtime/__manifest__');

define('THEME_URL',         HTTP_ROOT . '/themes/' . THEME);
define('THEME_PREVIEW_URL', HTTP_ROOT . '/themes/' . THEME . '/__editor__/runtime');

function startAction()
{
    rm_r(THEME_RUNTIME);

    cp_r(THEME_ROOT, THEME_RUNTIME, ['__editor__']);
    cp_r(THEME_PREVIEW, THEME_RUNTIME_PREVIEW);

    return ['status' => 'done'];
}

function exportAction()
{
    if ($data = getPost('fso')) {
        extractFiles(json_decode($data, true), THEME_RUNTIME);
    }

    if ('save' === getPost('method')) {
        rm_r(THEME_ROOT, ['__editor__']);
        rm_r(THEME_PREVIEW);

        cp_r(THEME_RUNTIME, THEME_ROOT, ['__preview__', '__manifest__']);
        cp_r(THEME_RUNTIME_PREVIEW, THEME_PREVIEW);
        if (is_dir(THEME_RUNTIME_MANIFEST)) {
            cp_r(THEME_RUNTIME_MANIFEST, MANIFEST);
        }
    }

    return ['status' => 'done'];
}

function zipAction()
{
    $source = getParam('sourceName');
    $target = getParam('targetName', $source);
    $includeEditor = (bool) json_decode(getParam('editor'));

    try {
        $sourcePath = $source ?
            THEMES . DS . $source :
            THEME_ROOT;
        $exclude = $includeEditor ? ['runtime'] : ['__editor__'];

        if (!file_exists($sourcePath)) {
            throw new Exception('Invalid source');
        }

        $result = ZipHelper::zip($sourcePath, $target, $exclude);
        if ($result['status'] === 'error') {
            return $result;
        } else {
            echo base64_encode(file_get_contents($result['path']));
            unlink($result['path']);
        }
    } catch (Exception $e) {
        return ['status' => 'error', 'message' => $e->getMessage()];
    }
}

function canRenameAction()
{
    $result = [
        'status' => 'done',
        'canRename' => false
    ];

    $themeName = getParam('themeName');

    if ($themeName) {
        $result['canRename'] = !file_exists(THEMES . DS . $themeName);
    }

    return $result;
}

function renameAction()
{
    $source = getParam('source');
    $target = getParam('target');

    $result = ['status' => 'done'];

    if (!$source || !file_exists(THEMES . DS . $source)) {
        $result = [
            'status' => 'error',
            'message' => 'Invalid source theme'
        ];
    } else if ($source === THEME) {
        $result = [
            'status' => 'error',
            'message' => 'Operation permitted'
        ];
    } else if ($target && file_exists(THEMES . DS . $target)) {
        $result = [
            'status' => 'error',
            'message' => 'Target already exists'
        ];
    } else if (!$target) {
        rm_r(THEMES . DS . $source, [], true);
    } else {
        rename(THEMES . DS . $source, THEMES . DS . $target);
    }

    return $result;
}

function copyAction()
{
    $source = getParam('source');
    $target = getParam('target');

    $result = ['status' => 'done'];

    if (!$source || !file_exists(THEMES . DS . $source)) {
        $result = [
            'status' => 'error',
            'message' => 'Invalid source theme'
        ];
    } else {
        cp_r(THEMES . DS . $source, THEMES . DS . getAvailableThemeName(THEMES, $target));
    }

    return $result;
}

function imageAction()
{
    $filename = getParam('filename');

    if (!$filename) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty file name'
        );
    } else {
        $uploadPath = THEME_RUNTIME . '/assets/images/' . $filename;

        try {
            $result = uploadChunk($uploadPath);
            if ($result['status'] === 'done') {
                $result['url'] = THEME_PREVIEW_URL . '/assets/images/' . $filename;
            }
        } catch (Exception $e) {
            $result = array(
                'status' => 'error',
                'message' => $e->getMessage(),
                'type' => 'permission'
            );
        }
    }

    return $result;
}

function themeAction()
{
    $filename = getParam('filename');

    if (!$filename) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty file name'
        );
    } else {
        $uploadPath = getcwd() . DS . $filename;

        try {
            $result = uploadChunk($uploadPath);

            if ($result['status'] === 'done') {
                $tempDir = getcwd() . DS . 'temp';
                $result = ZipHelper::unzip($uploadPath, $tempDir);

                if ($result['status'] === 'error') {
                    @rm_r($tempDir, [], true);
                    throw new Exception($result['message']);
                }

                $list = array_diff(scandir($tempDir), ['.', '..']);

                while (current($list) !== false && !is_dir($tempDir . DS . current($list))) {
                    next($list);
                }

                if (current($list) === false) {
                    @rm_r($tempDir, [], true);
                    throw new Exception('Wrong theme archive contents: ' . json_encode($list));
                }

                $theme = getAvailableThemeName(THEMES, current($list));
                cp_r($tempDir . DS . current($list), THEMES . DS . $theme);

                $manifest = THEMES . "/$theme/__editor__/themler.manifest";
                $editor = THEMES . "/$theme/__editor__/data/project/editor.json";

                if (file_exists($manifest) && file_exists($editor)) {
                    $cfg = json_decode(file_get_contents($editor));

                    if ($cfg && isset($cfg->version)) {
                        @mkdir(MANIFEST, 0777, true);
                        copy($manifest, MANIFEST . DS . $cfg->version . '.manifest');
                    }
                }

                rm_r($tempDir, [], true);
                @unlink($uploadPath);
            }
        } catch (Exception $e) {
            $result = array(
                'status' => 'error',
                'message' => $e->getMessage(),
                'type' => 'permission'
            );
        }
    }

    return $result;
}

function themesAction() {
    $result = [
        'status' => 'done',
        'themes' => []
    ];

    $list = array_diff(scandir(THEMES), ['.', '..']);

    foreach ($list as $theme) {
        $params = [];
        $editor = THEMES . DS . $theme . DS . '__editor__/data/project/editor.json';
        if (!file_exists($editor)) continue;

        if ($j = json_decode(file_get_contents($editor), true)) {
            if (!empty($j['version'])) {
                $params[] = 'ver=' . $j['version'];
            }
        }

        $result['themes'][$theme] = [
            'themeName' => $theme,
            'openUrl' => HTTP_ROOT . "/themes/$theme/__editor__/app.html" . ($params ? '?' . implode('&', $params) : ''),
            'thumbnailUrl' => HTTP_ROOT . "/themes/$theme/assets/images/preview.png",
            'isActive' => $theme === THEME
        ];
    }

    return $result;
}

dispatch();

//